"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var ZkSyncRewardsContractFactory_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZkSyncRewardsContractFactory = void 0;
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const zksync_web3_1 = require("zksync-web3");
const BaseContract_js_1 = require("../../implementations/BaseContract.js");
const GasUtils_1 = require("../../implementations/GasUtils");
const index_js_1 = require("../../interfaces/objects/index.js");
let ZkSyncRewardsContractFactory = ZkSyncRewardsContractFactory_1 = class ZkSyncRewardsContractFactory extends BaseContract_js_1.BaseContract {
    constructor(providerOrSigner, //SEANCHARLIE : needs to use ZkSync provider and wallet
    rewardType) {
        super(providerOrSigner, (0, objects_1.EVMContractAddress)(ethers_1.ethers.constants.AddressZero), // The rewards contract factory deploys a new contract, hence doesn't have a contract address
        index_js_1.ContractsAbis.ZkSyncERC721RewardAbi.abi);
        this.providerOrSigner = providerOrSigner;
        this.rewardType = rewardType;
        // Set the correct contract factory based on rewardTypeToDeploy
        this.contractFactory = new zksync_web3_1.ContractFactory(index_js_1.ContractsAbis.ZkSyncERC721RewardAbi.abi, index_js_1.ContractsAbis.ZkSyncERC721RewardAbi.bytecode, providerOrSigner);
        this.rewardTypeToDeploy = rewardType;
    }
    // function to deploy a new ERC721 reward contract on ZkSyncEra
    deployERC721Reward(name, symbol, baseURI, overrides) {
        return GasUtils_1.GasUtils.getGasFee(this.providerOrSigner).andThen((gasFee) => {
            const contractOverrides = {
                ...gasFee,
                ...overrides,
            };
            return this.writeToContractFactory("deploy", [name, symbol, baseURI], contractOverrides, true);
        });
    }
    estimateGasToDeployERC721Contract(name, symbol, baseURI) {
        return neverthrow_1.ResultAsync.fromPromise(this.providerOrSigner.estimateGas(this.contractFactory.getDeployTransaction(name, symbol, baseURI)), (e) => {
            return this.generateError(e, "Failed to wait() for contract deployment");
        }).map((estimatedGas) => {
            // Increase estimated gas buffer by 20%
            return estimatedGas.mul(120).div(100);
        });
    }
    generateContractSpecificError(msg, reason, e) {
        return new objects_1.RewardsFactoryError(msg, reason, e);
    }
    // Takes the factory's deploy function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContractFactory(functionName, functionParams, overrides, isDeployingContract) {
        return neverthrow_1.ResultAsync.fromPromise(this.contractFactory[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return this.generateError(e, `Unable to call ${functionName}()`);
        }).map((transactionResponse) => {
            var _a;
            // If we are deploying a contract, the deploy() call returns an ethers.Contract object and the txresponse is under the deployTransaction property
            return ZkSyncRewardsContractFactory_1.buildWrappedTransactionResponse(isDeployingContract == true
                ? transactionResponse.deployTransaction
                : transactionResponse, (0, objects_1.EVMContractAddress)(""), (0, objects_1.EVMAccountAddress)((_a = this.providerOrSigner) === null || _a === void 0 ? void 0 : _a.address), functionName, functionParams, index_js_1.ContractsAbis.ZkSyncERC721RewardAbi.abi);
        });
    }
};
exports.ZkSyncRewardsContractFactory = ZkSyncRewardsContractFactory;
exports.ZkSyncRewardsContractFactory = ZkSyncRewardsContractFactory = ZkSyncRewardsContractFactory_1 = __decorate([
    (0, inversify_1.injectable)() //SEANCHARLIE : do we want a specific zksync factory error?
    ,
    __metadata("design:paramtypes", [Object, String])
], ZkSyncRewardsContractFactory);
//# sourceMappingURL=ZkSyncRewardsContractFactory.js.map